@@ -1,9 +1,9 @@
 /******************************************************************************
  *
  * Name:        skdrv2nd.h
- * Project:     GEnesis, PCI Gigabit Ethernet Adapter
- * Version:     $Revision: 1.34.4.29 $
- * Date:        $Date: 2008/04/09 12:16:01 $
+ * Project:     Yukon/Yukon2, PCI Gigabit Ethernet Adapter
+ * Version:     $Revision: 1.44 $
+ * Date:        $Date: 2009/09/04 14:57:12 $
  * Purpose:     Second header file for driver and all other modules
  *
  ******************************************************************************/
@@ -52,17 +52,47 @@
 #include "h/skaddr.h"
 #include "h/skgesirq.h"
 #include "h/skcsum.h"
-#include "h/skrlmt.h"
 #include "h/skgedrv.h"
 #include "h/mvyexhw.h"
-#ifdef SK_ASF
-#include "h/skgeasf.h"
-#include "h/skgeasfconv.h"
+
+/* Disable ProcFS */
+#undef CONFIG_PROC_FS
+
+/* Enable SDK support */
+#ifdef MV_INCLUDE_SDK_SUPPORT
+// #define VERBOSE_ASF_FIFO_DBUG_RPINTS 
+// #define MV_SET_FW_IP_DRIVER
+#include "h/skspi.h"
+#include "h/skpflash.h"
+#include "h/skgeasfapi.h"
+#include "h/fwcommon.h"
+
+/* Set SDK parameters */
+#define MV_INCLUDE_FW_SDK_LINK_MAINTENANCE
+// #define MV_FW_SDK_LINK_MAINTENANCE_DEBUG
 #endif
 
 /* Required for 64 bit division */
 #include "asm/div64.h"
 
+/* Define the network status */
+#ifndef NETDEV_TX_BUSY
+#define NETDEV_TX_BUSY	1
+#endif
+
+#ifndef NETDEV_TX_OK
+#define NETDEV_TX_OK	0
+#endif
+
+/* Define the priv structure */
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,29)
+#define PRIV	dev->priv
+#define PPRIV	(DEV_NET*)dev->priv
+#else
+#define PRIV	netdev_priv(dev)
+#define PPRIV	netdev_priv(dev)
+#endif
+
 /* Defines for the poll cotroller */
 #if !defined(SK_NETDUMP_POLL)
 #define SK_NETDUMP_POLL
@@ -80,6 +110,23 @@
 #endif
 #endif
 
+/******************************************************************************
+ *
+ * SDK driver defines (CHANGE IT!!!!)
+ *
+ ******************************************************************************/
+#ifdef SK_NO_RLMT
+#undef SKGE_RLMT
+#define SKGE_RLMT SKGE_DRV
+#undef SK_RLMT_LINK_UP
+#define SK_RLMT_LINK_UP SK_DRV_LINK_UP
+#undef SK_RLMT_LINK_DOWN
+#define SK_RLMT_LINK_DOWN SK_DRV_LINK_DOWN
+#define SkRlmtEvent(a,b,c,d)	SkDrvEvent(a,b,c,d)
+#endif
+
+#define SKGE_ASF	14	/* ASF Event Class */
+
 /******************************************************************************
  *
  * Generic driver defines
@@ -108,26 +155,24 @@
  * use those defines for a compile-in version of the driver instead
  * of command line parameters
  */
-// #define LINK_SPEED_A	{"Auto",}
-// #define LINK_SPEED_B	{"Auto",}
-// #define AUTO_NEG_A	{"Sense",}
-// #define AUTO_NEG_B	{"Sense"}
-// #define DUP_CAP_A	{"Both",}
-// #define DUP_CAP_B	{"Both",}
-// #define FLOW_CTRL_A	{"SymOrRem",}
-// #define FLOW_CTRL_B	{"SymOrRem",}
-// #define ROLE_A	{"Auto",}
-// #define ROLE_B	{"Auto",}
-// #define PREF_PORT	{"A",}
-// #define CON_TYPE 	{"Auto",}
-// #define RLMT_MODE	{"CheckLinkState",}
+// #define LINK_SPEED_A {"Auto",}
+// #define LINK_SPEED_B {"Auto",}
+// #define AUTO_NEG_A   {"Sense",}
+// #define AUTO_NEG_B   {"Sense"}
+// #define DUP_CAP_A    {"Both",}
+// #define DUP_CAP_B    {"Both",}
+// #define FLOW_CTRL_A  {"SymOrRem",}
+// #define FLOW_CTRL_B  {"SymOrRem",}
+// #define ROLE_A       {"Auto",}
+// #define ROLE_B       {"Auto",}
+// #define CON_TYPE     {"Auto",}
+// #define WOL_TYPE     {"Auto",}
 
 #ifdef Y2_RECOVERY
 #define CHECK_TRANSMIT_TIMEOUT
 #define Y2_RESYNC_WATERMARK     1000000L
 #endif
 
-
 /******************************************************************************
  *
  * Generic ISR defines
@@ -147,8 +192,6 @@
  *
  ******************************************************************************/
 
-extern SK_MBUF *SkDrvAllocRlmtMbuf(SK_AC*, SK_IOC, unsigned);
-extern void SkDrvFreeRlmtMbuf(SK_AC*, SK_IOC, SK_MBUF*);
 extern SK_U64 SkOsGetTime(SK_AC*);
 extern int SkPciReadCfgDWord(SK_AC*, int, SK_U32*);
 extern int SkPciReadCfgWord(SK_AC*, int, SK_U16*);
@@ -159,28 +202,7 @@ extern int SkPciWriteCfgByte(SK_AC*, int, SK_U8);
 extern int SkDrvEvent(SK_AC*, SK_IOC IoC, SK_U32, SK_EVPARA);
 extern int SkDrvEnterDiagMode(SK_AC *pAc);
 extern int SkDrvLeaveDiagMode(SK_AC *pAc);
-#ifdef USE_ASF_DASH_FW
-extern void	SetRamAddr(SK_AC *, int, int, int);
-extern SK_U8 AccessRamBuf(SK_AC *, int, SK_BOOL);
-#endif
-
-/******************************************************************************
- *
- * Linux specific RLMT buffer structure (SK_MBUF typedef in skdrv1st)!
- *
- ******************************************************************************/
-
-struct s_DrvRlmtMbuf {
-	SK_MBUF         *pNext;    /* Pointer to next RLMT Mbuf.       */
-	SK_U8           *pData;    /* Data buffer (virtually contig.). */
-	unsigned         Size;     /* Data buffer size.                */
-	unsigned         Length;   /* Length of packet (<= Size).      */
-	SK_U32           PortIdx;  /* Receiving/transmitting port.     */
-#ifdef SK_RLMT_MBUF_PRIVATE
-	SK_RLMT_MBUF     Rlmt;     /* Private part for RLMT.           */
-#endif
-	struct sk_buff  *pOs;      /* Pointer to message block         */
-};
+extern int spi_timer(SK_AC *pAC, unsigned int t);
 
 /******************************************************************************
  *
@@ -194,19 +216,10 @@ struct s_DrvRlmtMbuf {
 #define SK_PNMI_HUNDREDS_SEC(t) ((((unsigned long)t)*100)/(SK_TICKS_PER_SEC))
 #endif
 
-#ifdef USE_ALT_TIMING
-#define __RDTSCLL(val) __asm__ __volatile__("rdtsc": "=A" (val))
-
-#define SkOsGetTimeCurrent(pAC, pUsec) { \
-	SK_U64 currentTime; \
-	__RDTSCLL(currentTime); \
-	do_div(currentTime, cpu_khz); \
-	do_div(currentTime,(SK_U64)((SK_U32)1000 / SK_TICKS_PER_SEC)); \
-	*pUsec = currentTime; \
-}
+#ifdef MV_INCLUDE_SDK_SUPPORT
+#define SK_FLASH_RESCHED cond_resched
 #endif
 
-#ifndef USE_ALT_TIMING
 #define SkOsGetTimeCurrent(pAC, pUsec) {\
 	static struct timeval prev_t; \
 	struct timeval t;\
@@ -222,7 +235,6 @@ struct s_DrvRlmtMbuf {
 	} \
 	*pUsec = ((t.tv_sec*100L)+(t.tv_usec/10000));\
 }
-#endif
 
 /******************************************************************************
  *
@@ -236,23 +248,32 @@ struct s_DrvRlmtMbuf {
 #define	SK_IOCTL_PRESETMIB   (SK_IOCTL_BASE + 2)
 #define	SK_IOCTL_GEN         (SK_IOCTL_BASE + 3)
 #define	SK_IOCTL_DIAG        (SK_IOCTL_BASE + 4)
-#ifdef USE_ASF_DASH_FW
-#define	SK_IOCTL_IPTOFW      (SK_IOCTL_BASE + 10)
-#define SK_IOCTL_PATTERNTOFW (SK_IOCTL_BASE + 11)
-#define SK_IOCTL_READASFFIFO (SK_IOCTL_BASE + 12)
-#define SK_IOCTL_LINKTOFW    (SK_IOCTL_BASE + 13)
-#define SK_IOCTL_GETLINKCAP  (SK_IOCTL_BASE + 14)
 
+/* Embedded SDK driver ioctl */
+#define SK_IOCTL_GETLINKCAP  (SK_IOCTL_BASE + 14)
 #define SK_AUTONEG_SUCCESS 0
 #define SK_AUTONEG_FAIL    1
 #define SK_AUTONEG_OFF     2
 #define SK_AUTONEG_NOLINK  3
 
-#define SK_ST_FIFOTYPE		3
-#define SK_ST_BUFADDR_LOW	0xc5
-#define SK_ST_BUFADDR_HIGH	0x0
-#define SK_ST_BUFSIZE		191
-#endif
+/* Firmware ioctl defines and commands */
+#define SK_IOCTL_APPTOFW     (SK_IOCTL_BASE + 5)
+#define SK_IOCTL_APPTODRV    (SK_IOCTL_BASE + 6)
+#define SK_IOCTL_FW_SEND	1
+#define SK_IOCTL_FW_RECEIVE	2
+#define SK_IOCTL_COMMAND	5
+
+typedef struct s_FWIOCTL SK_GE_FWIOCTL;
+struct s_FWIOCTL {
+	unsigned int	Command;
+	unsigned int	Len;
+	char  __user *	pData;
+};
+
+typedef struct _DrvCmd SK_GE_CMDIOCTL;
+struct _DrvCmd {
+	SK_U32   Command;
+};
 
 typedef struct s_IOCTL SK_GE_IOCTL;
 struct s_IOCTL {
@@ -266,8 +287,8 @@ struct s_IOCTL {
  *
  ******************************************************************************/
 
-#define TX_RING_SIZE  (24*1024)			/* GEnesis/Yukon */
-#define RX_RING_SIZE  (24*1024)			/* GEnesis/Yukon */
+#define TX_RING_SIZE  (24*1024)			/* Yukon */
+#define RX_RING_SIZE  (24*1024)			/* Yukon */
 #define RX_MAX_NBR_BUFFERS   128		/* Yukon-EC/-II */
 #define TX_MAX_NBR_BUFFERS   128		/* Yukon-EC/-II */
 #define MAXIMUM_LOW_ADDRESS 0xFFFFFFFF		/* Max. low address */
@@ -360,8 +381,13 @@ struct s_TxD {
 	SK_U32           VDataHigh;     /* Transmit Buffer Addr, high dword   */
 	SK_U32           FrameStat;     /* Transmit Frame Status Word         */
 	SK_U32           TcpSumOfs;     /* Reserved / TCP Sum Offset          */
+#ifndef SK_USE_REV_DESC
+	SK_U16           TcpSumWr;      /* TCP Sum Write                      */
+	SK_U16           TcpSumSt;      /* TCP Sum Start                      */
+#else
 	SK_U16           TcpSumSt;      /* TCP Sum Start                      */
 	SK_U16           TcpSumWr;      /* TCP Sum Write                      */
+#endif
 	SK_U32           TcpReserved;   /* not used                           */
 	TXD             *pNextTxd;      /* Pointer to next Txd                */
 	struct sk_buff  *pMBuf;         /* Pointer to Linux' socket buffer    */
@@ -699,37 +725,34 @@ struct s_queue {
 
 #define IRQ_HWE_MASK	(IS_ERR_MSK)		/* enable all HW irqs */
 #define Y2_IRQ_HWE_MASK	(Y2_HWE_ALL_MSK)	/* enable all HW irqs */
+#define Y2_IS_BASE      ((Y2_IS_HW_ERR) | (Y2_IS_STAT_BMU))
 
 typedef struct s_DevNet DEV_NET;
 
 struct s_DevNet {
-	struct		proc_dir_entry *proc;
-	int		PortNr;
-	int		NetNr;
-	char		InitialDevName[20];
-	char		CurrentName[20];
-	SK_BOOL		NetConsoleMode;
+	struct      proc_dir_entry *proc;
+	int         PortNr;
+	int         NetNr;
+	char        InitialDevName[20];
+	char        CurrentName[20];
+	SK_BOOL     NetConsoleMode;
 #ifdef Y2_RECOVERY
-	struct 		timer_list KernelTimer;	/* Kernel timer struct  */
-	int		TransmitTimeoutTimer; 	/* Transmit timer       */
-	SK_BOOL		TimerExpired;		/* Transmit timer       */
-	SK_BOOL		InRecover;		/* Recover flag		*/
+	struct      timer_list KernelTimer;	/* Kernel timer struct  */
+	int         TransmitTimeoutTimer; 	/* Transmit timer       */
+	SK_BOOL     TimerExpired;		/* Transmit timer       */
+	SK_BOOL     InRecover;		/* Recover flag		*/
 #ifdef Y2_RX_CHECK
-	SK_U32 		PreviousMACFifoRP;	/* Backup of the FRP    */
-	SK_U32 		PreviousMACFifoRLev;	/* Backup of the FRL    */
-	SK_U32 		PreviousRXFifoRP;	/* Backup of the RX FRP */
-	SK_U8 		PreviousRXFifoRLev;	/* Backup of the RX FRL */
-	SK_U32		LastJiffies;		/* Backup of the jiffies*/
+	SK_U32      PreviousMACFifoRP;	/* Backup of the FRP    */
+	SK_U32      PreviousMACFifoRLev;	/* Backup of the FRL    */
+	SK_U32      PreviousRXFifoRP;	/* Backup of the RX FRP */
+	SK_U8       PreviousRXFifoRLev;	/* Backup of the RX FRL */
+	SK_U32      LastJiffies;		/* Backup of the jiffies*/
 #endif
 #endif
-	SK_AC		*pAC;
-	struct 		timer_list ProcfsTimer;	/* Procfs timer struct  */
-#ifdef USE_ASF_DASH_FW
-	struct 		timer_list LLCTimer;	/* LLC timer struct  */
-	struct 		timer_list WakeupTimer;	/* Wakeup packet timer struct  */
-#endif
+	SK_AC       *pAC;
+	struct      timer_list ProcfsTimer;	/* Procfs timer struct  */
 #if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,23)
-	struct		napi_struct napi;	/* NAPI structure */
+	struct      napi_struct napi;	/* NAPI structure */
 #endif
 };  
 
@@ -749,7 +772,7 @@ struct s_TxPort {                       /* the transmit descriptor rings */
 	int             TxdRingPrevFree;/* previously # of free entrys   */
 	int             TxdRingFree;    /* # of free entrys              */
 	spinlock_t      TxDesRingLock;  /* serialize descriptor accesses */
-	SK_IOC		HwAddr;         /* bmu registers address         */
+	SK_IOC          HwAddr;         /* bmu registers address         */
 	int             PortIndex;      /* index number of port (0 or 1) */
 	SK_PACKET      *TransmitPacketTable;
 	SK_LE_TABLE     TxALET;         /* tx (async) list element table */
@@ -759,7 +782,7 @@ struct s_TxPort {                       /* the transmit descriptor rings */
 	SK_PKT_QUEUE    TxSQ_waiting;
 	SK_PKT_QUEUE    TxAQ_working;
 	SK_PKT_QUEUE    TxSQ_working;
-	unsigned	LastDone;
+	unsigned        LastDone;
 };
 
 typedef struct s_RxPort	RX_PORT;
@@ -855,7 +878,10 @@ typedef struct s_WakeOnLanInfo {
 	SK_U32     ConfiguredWolOptions;        /* e.g. WAKE_PHY...         */
 } WOL_INFO;
 
+/* Alloc flags */
 #define SK_ALLOC_IRQ	0x00000001
+#define SK_ALLOC_MSI    0x00000002
+
 #define	DIAG_ACTIVE		1
 #define	DIAG_NOTACTIVE		0
 
@@ -876,41 +902,28 @@ struct s_AC  {
 	SK_I2C                   I2c;           /* I2C relevant data structure*/
 	SK_ADDR                  Addr;          /* for Address module         */
 	SK_CSUM                  Csum;          /* for checksum module        */
-	SK_RLMT                  Rlmt;          /* for rlmt module            */
-#ifdef SK_ASF
-	SK_ASF_DATA              AsfData;
-	unsigned char            IpAddr[4];
-#endif
-#ifdef USE_ASF_DASH_FW
-	unsigned char            IpV6Addr[16*7];
-	unsigned int             ForceFWIPUpdate;
-	unsigned int             RecvNewPattern;
-	DRIVER_INTERFACE         NewPatternDef;
+#ifdef MV_INCLUDE_SDK_SUPPORT
+	SK_U64                   timebuf;
+	SK_FWOS                  FwOs;
+	SK_FWCOMMON              FwCommon;
+	SK_FWAPP                 FwApp;
+	SK_SPI                   spi;           /* structure for SPI flash */
+	SK_PFL                   pfl;
 	SK_U32                   RamAddr;
-	int                      RamSelect;
-	unsigned int             ReturningFromSuspend; /* Just resumed? Reset by ASF */
-	unsigned int             ReceivedPacket;
-	unsigned int             SendWolPattern;
-	unsigned int             SendLLCPacket;
-	unsigned char            SuspendLinkSpeed;
-	unsigned char            SuspendDuplexMode;
-	unsigned char            LinkSpeed;
-	unsigned char            LinkSpeedSet;
-	unsigned char            SetIFFRunning;
+	SK_U32                   RamSelect;
 #endif
 	spinlock_t               SlowPathLock;  /* Normal IRQ lock            */
-	spinlock_t               InitLock;	/* Init lock                  */
+	spinlock_t               InitLock;      /* Init lock                  */
 	spinlock_t               TxQueueLock;   /* TX Queue lock              */
 	SK_PNMI_STRUCT_DATA      PnmiStruct;    /* struct for all Pnmi-Data   */
-	int                      RlmtMode;      /* link check mode to set     */
 	int                      RlmtNets;      /* Number of nets             */
 	SK_IOC                   IoBase;        /* register set of adapter    */
 	int                      BoardLevel;    /* level of hw init (0-2)     */
 	char                     DeviceStr[80]; /* adapter string from vpd    */
-	SK_U32                   AllocFlag;     /* alloc flag of resources    */
 	struct pci_dev          *PciDev;        /* for access to pci cfg space*/
 	SK_U32                   PciDevId;      /* pci device id              */
 	struct SK_NET_DEVICE    *dev[2];        /* pointer to device struct   */
+	struct SK_NET_DEVICE    *IrqDev;        /* pointer to device struct   */
 	char                     Name[30];      /* driver name                */
 	struct SK_NET_DEVICE    *Next;          /* link all devs for cleanup  */
 	struct net_device_stats  stats;         /* linux 'netstat -i' stats   */
@@ -928,45 +941,63 @@ struct s_AC  {
 	int                      RxDescrPerRing;/* # of descriptors RX ring   */
 	caddr_t                  pDescrMem;     /* Ptr to the descriptor area */
 	dma_addr_t               pDescrMemDMA;  /* PCI DMA address of area    */
-	SK_U32			 PciState[16];  /* PCI state */
+	SK_U32                   PciState[16];  /* PCI state */
 	TX_PORT                  TxPort[SK_MAX_MACS][2];
 	RX_PORT                  RxPort[SK_MAX_MACS];
 	SK_LE_TABLE              StatusLETable; 
 	unsigned                 SizeOfAlignedLETables;	
 	spinlock_t               SetPutIndexLock;
 	int                      MaxUnusedRxLeWorking;
-        int                      InterfaceUp[2];
+	int                      InterfaceUp[2];
 	unsigned int             CsOfs1;        /* for checksum calculation   */
 	unsigned int             CsOfs2;        /* for checksum calculation   */
 	SK_U32                   CsOfs;         /* for checksum calculation   */
 	SK_BOOL                  CheckQueue;    /* check event queue soon     */
 	DIM_INFO                 DynIrqModInfo; /* all data related to IntMod */
 	WOL_INFO                 WolInfo;       /* all info regarding WOL     */
-	int                      ChipsetType;   /* 0=GENESIS; 1=Yukon         */
+	int                      ChipsetType;   /* 1=Yukon, 2=Yukon2          */
 	SK_BOOL                  LowLatency;    /* LowLatency optimization on?*/
+	SK_BOOL                  MsiIrq;        /* MSI Irq handling on?       */
 	SK_U32                   TxModeration;  /* TxModeration optimization  */
 	SK_U32                   DiagModeActive;/* is diag active?            */
 	SK_BOOL                  DiagFlowCtrl;  /* for control purposes       */
 	SK_PNMI_STRUCT_DATA      PnmiBackup;    /* backup structure for PNMI  */
 	SK_BOOL                  WasIfUp[SK_MAX_MACS];
+	SK_U32                   WolSpeedType;   /* Wol speed type */
+	SK_U32                   WolDuplexType;  /* Wol type; TRUE==FD; FALSE==HD */
 	char                     *pVirtMemAddr;
+	wait_queue_head_t        msi_wait;
+	SK_U32                   AllocFlag;     /* alloc flag of resources    */
 #ifdef USE_TIST_FOR_RESET
-	int			 AdapterResetState;
-	SK_U32			 MinTistLo;
-	SK_U32			 MinTistHi;
+	int                      AdapterResetState;
+	SK_U32                   MinTistLo;
+	SK_U32                   MinTistHi;
 #endif
 #ifdef Y2_RECOVERY
 	int                      LastPort;       /* port for curr. handled rx */
-        int                      LastOpc;        /* last rx LEs opcode	      */
+	int                      LastOpc;        /* last rx LEs opcode	      */
 #endif
 #ifdef Y2_SYNC_CHECK
 	unsigned long            FramesWithoutSyncCheck; /* since last check  */
 #endif
+	SK_BOOL                  RequestedIrq;
 	unsigned int             InterruptSource;
+	SK_BOOL                  Suspended;
+#ifdef MV_INCLUDE_SDK_SUPPORT
+	SK_BOOL                  FwRun;
+	SK_BOOL                  FwState;
+	SK_U32                   SdkQH;
+#ifdef MV_INCLUDE_FW_SDK_LINK_MAINTENANCE
+	SK_BOOL                  UseLinkMaintenance;
+	SK_U16                   DriverLinkStat;
+#endif
+	spinlock_t               FwFifoLock;
+	SK_U32                   *pFwBuffer;
+	int                      FwBufferLen;
+	SK_U64                   TimeBuf;
+#endif
 };
 
-
-
 #endif
 
 /*******************************************************************************
