@@ -2,8 +2,8 @@
  *
  * Name:	skgeinit.h
  * Project:	Gigabit Ethernet Adapters, Common Modules
- * Version:	$Revision: 2.79 $
- * Date:	$Date: 2008/04/15 15:37:57 $
+ * Version:	$Revision: 2.90 $
+ * Date:	$Date: 2010/01/19 14:06:02 $
  * Purpose:	Structures and prototypes for the GE Init Module
  *
  ******************************************************************************/
@@ -87,7 +87,7 @@ extern "C" {
 #define SK_RB_LLPP_S	(10 * 1024)	/* Lower Level for small Queues */
 #define SK_RB_LLPP_B	(16 * 1024)	/* Lower Level for big Queues */
 
-/* Threshold values for Yukon-EC Ultra and Extreme*/
+/* Threshold values for Yukon-EC Ultra and Extreme */
 #define SK_ECU_ULPP		0x0080	/* Upper Pause Threshold (multiples of 8) */
 #define SK_DEV521_ULPP	0x00c4	/* Upper Pause Threshold for Dev. 5.21 */
 #define SK_ECU_LLPP		0x0060	/* Lower Pause Threshold (multiples of 8) */
@@ -95,6 +95,8 @@ extern "C" {
 #define SK_ECU_TXFF_LEV	0x01a0	/* Tx BMU FIFO Level */
 #define SK_ECU_JUMBO_WM	0x0400	/* Jumbo Mode Watermark */
 
+#define SK_EXT_ULPP		0x05c0	/* Upper Pause Threshold (multiples of 8) */
+
 #ifndef SK_BMU_RX_WM
 #define SK_BMU_RX_WM		0x600	/* BMU Rx Watermark */
 #endif
@@ -288,9 +290,6 @@ extern "C" {
 #define SK_PCIX_BUS		BIT_1S		/* PCI-X bus */
 #define SK_PEX_BUS		BIT_2S		/* PCI-Express bus */
 
-/* Default receive frame limit for Workaround of XMAC Errata */
-#define SK_DEF_RX_WA_LIM	SK_CONSTU64(100)
-
 /* values for GILedBlinkCtrl (LED Blink Control) */
 #define SK_ACT_LED_BLINK	BIT_0S	/* Activity LED blinking */
 #define SK_DUP_LED_NORMAL	BIT_1S	/* Duplex LED normal */
@@ -327,8 +326,11 @@ extern "C" {
 #define HWF_TX_IP_ID_INCR_ON	0x00200000UL	/* Enable Tx IP ID Increment */
 #define HWF_ADV_CSUM_SUPPORT	0x00100000UL	/* Sel Csum of IP and TCP/UDP */
 #define HWF_PSM_SUPPORTED		0x00080000UL	/* Power State Manager support*/
-#define HWF_NEW_FLOW_CONTROL	0x00040000UL	/* New Flow-Control support*/
+#define HWF_NEW_FLOW_CONTROL	0x00040000UL	/* New Flow-Control support */
 #define HWF_HW_WOL_ENABLE		0x00020000UL	/* Enable HW WOL */
+#define HWF_D0_CLK_GAT_ENABLE	0x00010000UL	/* Enable D0 Clock Gating */
+#define HWF_TRAFO_LESS_ENABLE	0x00008000UL	/* Enable PHY Trafo-less */
+#define HWF_PHY_CLASS_A_100BT	0x00004000UL	/* Enable PHY Class 100Base-T */
 
 
 /*-RMV- DWORD 1: Deviations (all in use) */
@@ -340,7 +342,7 @@ extern "C" {
 #define HWF_WA_DEV_4217			0x10400000UL	/*-RMV- 4.217 (PCI-E blockage) */
 #define HWF_WA_DEV_4200			0x10200000UL	/*-RMV- 4.200 (D3 Blue Screen)*/
 #define HWF_WA_DEV_4185CS		0x10100000UL	/*-RMV- 4.185 (ECU 100 CS cal)*/
-#define HWF_WA_DEV_4185			0x10080000UL	/*-RMV- 4.185 (ECU Tx h check)*/
+#define HWF_WA_DEV_4185			0x10080000UL	/*-RMV- 4.185 (ECU Tx hang) */
 #define HWF_WA_DEV_4167			0x10040000UL	/*-RMV- 4.167 (Rx OvSize Hang)*/
 #define HWF_WA_DEV_4152			0x10020000UL	/*-RMV- 4.152 (RSS issue) */
 #define HWF_WA_DEV_4115			0x10010000UL	/*-RMV- 4.115 (Rx MAC FIFO) */
@@ -365,6 +367,7 @@ extern "C" {
 #define HWF_WA_COMA_MODE		0x10000001UL	/*-RMV- Coma Mode WA req */
 
 /*-RMV- DWORD 2: Deviations */
+#define HWF_WA_DEV_547			0x20001000UL	/*-RMV- 5.47 (IRQ + clk gating)*/
 #define HWF_WA_DEV_548			0x20000800UL	/*-RMV- 5.48 (IPv4 Header CS) */
 #define HWF_WA_DEV_4216			0x20000400UL	/*-RMV- 4.216 (fragm. IPV4+RSS)*/
 #define HWF_WA_DEV_519			0x20000200UL	/*-RMV- 5.19 (MACsec: unexp. LE)*/
@@ -446,7 +449,10 @@ extern "C" {
 	 (pAC)->GIni.GIChipId >= CHIP_ID_YUKON_SUPR)
 
 #define HW_MS_TO_TICKS(pAC, MsTime) \
-	((MsTime) * (62500L/100) * (pAC)->GIni.GIHstClkFact)
+	((MsTime) * (1000L * (pAC)->GIni.GIHstClkFact * SK_FACT_62) / SK_FACT_100)
+
+#define HW_TICKS_TO_USEC(pAC, HwTicks) \
+	((HwTicks) / ((((pAC)->GIni.GIHstClkFact) * SK_FACT_62) / SK_FACT_100))
 
 #define HW_IS_EXT_LE_FORMAT(pAC)	((pAC)->GIni.GIExtLeFormat)
 #define HW_IS_NEW_ASF_FORMAT(pAC)	((pAC)->GIni.GIExtLeFormat)
@@ -460,8 +466,10 @@ extern "C" {
 	 (pAC)->GIni.GIChipId == CHIP_ID_YUKON_FE)
 
 #define HW_NUM_OF_PATTERN(pAC)	((pAC)->GIni.GINumOfPattern)
-#define HW_OTP_SUPPORTED(pAC)	((pAC)->GIni.GIChipId == CHIP_ID_YUKON_FE_P || \
-								 (pAC)->GIni.GIChipId == CHIP_ID_YUKON_UL_2)
+
+#define HW_OTP_SUPPORTED(pAC)						\
+	((pAC)->GIni.GIChipId == CHIP_ID_YUKON_FE_P ||	\
+	 (pAC)->GIni.GIChipId >= CHIP_ID_YUKON_UL_2)
 
 /* general CSUM */
 /* flag to distinguish between old style and new style csum offload */
@@ -493,11 +501,11 @@ extern "C" {
 #define HW_SUP_TX_CSUM_OPT_TCPIPV6(pAC)	((pAC)->GIni.GIExtLeFormat)
 #define HW_SUP_TX_CSUM_UDPIPV6(pAC)		((pAC)->GIni.GIExtLeFormat)
 
-/* Tx CSUM special flag for FiFo size */
-#define HW_SUP_TX_CSUM_FOR_JUMBO(pAC)	(		\
-	(pAC)->GIni.GIJumTcpSegSup ||				\
-	(pAC)->GIni.GIChipId == CHIP_ID_YUKON ||	\
-	(pAC)->GIni.GIChipId == CHIP_ID_YUKON_LITE)
+/* Tx CSUM special flag for FIFO size */
+#define HW_SUP_TX_CSUM_FOR_JUMBO(pAC)			\
+	((pAC)->GIni.GIJumTcpSegSup ||				\
+	 (pAC)->GIni.GIChipId == CHIP_ID_YUKON ||	\
+	 (pAC)->GIni.GIChipId == CHIP_ID_YUKON_LITE)
 
 /* LSO */
 #define HW_SUP_LSOV1(pAC)				((pAC)->GIni.GIYukon2)
@@ -509,13 +517,13 @@ extern "C" {
 #define HW_SUP_LSO_FOR_JUMBO(pAC)		((pAC)->GIni.GIJumTcpSegSup)
 
 /* RSS */
-#define HW_SUP_IPV4_RSS(pAC)			((pAC)->GIni.GIYukon2 &&			\
-										  !HW_FEATURE(pAC, HWF_WA_DEV_4152))
+#define HW_SUP_IPV4_RSS(pAC)									\
+	((pAC)->GIni.GIYukon2 && !HW_FEATURE(pAC, HWF_WA_DEV_4152))
 
-/* 
+/*
  * IPV6_RSS is supported since Yukon-Extreme B0
- * To avoid that IPV6 RSS is executed on Extreme A0 (which is opsolete already)
- * the HW_SUM_IPV6_RSS condition needs to be combined with the term 
+ * To avoid that IPV6 RSS is executed on Extreme A0 (which is obsolete already)
+ * the HW_SUM_IPV6_RSS condition needs to be combined with the term
  * "&& !HW_FEATURE(pAC, HWF_WA_DEV_LIM_IPV6_RSS)"
  */
 #define HW_SUP_IPV6_RSS(pAC)			((pAC)->GIni.GIExtLeFormat)
@@ -525,12 +533,26 @@ extern "C" {
 	((pAC)->GIni.GIChipId == CHIP_ID_YUKON_EX ||	\
 	 (pAC)->GIni.GIChipId == CHIP_ID_YUKON_SUPR)
 
+/* AVB */
+#define HW_SUP_AVB(pAC)							\
+	((pAC)->GIni.GIChipId == CHIP_ID_YUKON_OPT)
+
+/* PTP */
+#define HW_SUP_PTP(pAC)							\
+	((pAC)->GIni.GIChipId == CHIP_ID_YUKON_OPT)
+
+#define AVB_LE_Q_CNT	8   /* Number of available AVB LE queues */
+#define AVB_MAC_Q_CNT	5	/* Number of available AVB MAC FIFO queues */
+#define AVB_CLS_CNT		2	/* Number of available AVB classes */
+
 /* VPD Support */
-#define HW_SUP_VPD(pAC)			((pAC)->GIni.GIChipId != CHIP_ID_YUKON_FE_P)
+#define HW_SUP_VPD(pAC)							\
+	((pAC)->GIni.GIChipId != CHIP_ID_YUKON_FE_P)
 
 /* SPI PROM Support */
-#define HW_SUP_SPI_PROM(pAC)	((pAC)->GIni.GIChipId != CHIP_ID_YUKON_FE_P &&	\
-								 (pAC)->GIni.GIChipId != CHIP_ID_YUKON_FE)
+#define HW_SUP_SPI_PROM(pAC)						\
+	((pAC)->GIni.GIChipId != CHIP_ID_YUKON_FE_P &&	\
+	 (pAC)->GIni.GIChipId != CHIP_ID_YUKON_FE)
 
 /* Battery Power Management */
 #define HW_NO_BAT_POW_MANAGEMENT(pAC)				\
@@ -578,24 +600,13 @@ typedef struct s_GeMacFunc {
 /*
  * Port Structure
  */
-typedef	struct s_GePort {
+typedef struct s_GePort {
 #ifndef SK_DIAG
 	SK_TIMER	PWaTimer;	/* Workaround Timer */
-	SK_TIMER	HalfDupChkTimer;
 	SK_TIMER	PPauseTimer;/* Pause Packet Timer */
 	SK_U32		PPauseTxCnt;/* Pause Packet Counter (Tx) */
 	SK_BOOL		PPauseFlag;	/* Pause Packet Flag */
 #endif /* !SK_DIAG */
-#ifndef SK_SLIM
-	SK_U32	PPrevShorts;	/* Previous Short Counter checking */
-	SK_U32	PPrevFcs;		/* Previous FCS Error Counter checking */
-	SK_U64	PPrevRx;		/* Previous RxOk Counter checking */
-	SK_U64	PRxLim;			/* Previous RxOk Counter checking */
-	SK_U64	LastOctets;		/* For half duplex hang check */
-	int		PLinkResCt;		/* Link Restart Counter */
-	int		PAutoNegTimeOut;/* Auto-negotiation timeout current value */
-	int		PAutoNegTOCt;	/* Auto-negotiation Timeout Counter */
-#endif
 	int		PRxQSize;		/* Port Rx Queue Size in kB */
 	int		PXSQSize;		/* Port Synchronous  Transmit Queue Size in kB */
 	int		PXAQSize;		/* Port Asynchronous Transmit Queue Size in kB */
@@ -616,15 +627,10 @@ typedef	struct s_GePort {
 	int		PPortUsage;		/* Driver Port Usage */
 	SK_U16	PhyId1;			/* PHY Id1 on this port */
 	SK_U16	PhyAddr;		/* MDIO/MDC PHY address */
-	SK_U16	PGmANegAdv;		/* Saved GPhy AutoNegAdvertisment register */
 	SK_BOOL	PHWLinkUp;		/* The hardware Link is up (wiring) */
+	SK_BOOL	PPhyQLink;		/* PHY Quick Link detected */
+	SK_BOOL	PRevAutoNeg;	/* PHY Reverse Auto-negotiation */
 	SK_BOOL	PCheckPar;		/* Do we check for parity errors ? */
-#ifndef SK_SLIM
-	SK_U16	PSsave;			/* Saved PHY status word */
-	SK_U16	PIsave;			/* Saved Interrupt status word */
-	SK_BOOL	PLinkBroken;	/* Is Link broken ? */
-	SK_BOOL	HalfDupTimerActive;
-#endif
 	SK_U8	PLinkModeConf;	/* Link Mode configured */
 	SK_U8	PLinkMode;		/* Link Mode currently used */
 	SK_U8	PLinkModeStatus;/* Link Mode Status */
@@ -661,11 +667,85 @@ typedef	struct s_GePort {
 #endif
 } SK_GEPORT;
 
+/*
+ * Configuration Data Structure
+ */
+typedef union s_Cfg{
+	struct {
+		SK_U32 ValidConfig:1;
+		SK_U32 MemoryType:1;
+		SK_U32 reserved_1:2;
+		SK_U32 LED_Scheme:4;
+		SK_U32 WakeFromS5:1;
+		SK_U32 HW_WoLAN:1;
+		SK_U32 reserved_2:2;
+		SK_U32 EnergyScheme:4;
+		SK_U32 reserved_3:4;
+		SK_U32 AutoNegScheme:2;
+		SK_U32 MonSpeedStep:1;
+		SK_U32 SmartSpeed:1;
+		SK_U32 WakeUpSpeed:2;
+		SK_U32 DynClockGating:1;
+		SK_U32 reserved_4:5;
+	} fields;
+
+	struct {
+		SK_U32 D00:1;
+		SK_U32 D01:1;
+		SK_U32 D02:1;
+		SK_U32 D03:1;
+		SK_U32 D04:1;
+		SK_U32 D05:1;
+		SK_U32 D06:1;
+		SK_U32 D07:1;
+		SK_U32 D08:1;
+		SK_U32 D09:1;
+		SK_U32 D10:1;
+		SK_U32 D11:1;
+		SK_U32 D12:1;
+		SK_U32 D13:1;
+		SK_U32 D14:1;
+		SK_U32 D15:1;
+		SK_U32 D16:1;
+		SK_U32 D17:1;
+		SK_U32 D18:1;
+		SK_U32 D19:1;
+		SK_U32 D20:1;
+		SK_U32 D21:1;
+		SK_U32 D22:1;
+		SK_U32 D23:1;
+		SK_U32 D24:1;
+		SK_U32 D25:1;
+		SK_U32 D26:1;
+		SK_U32 D27:1;
+		SK_U32 D28:1;
+		SK_U32 D29:1;
+		SK_U32 D30:1;
+		SK_U32 D31:1;
+	} bits;
+
+	SK_U32 ApplicConfigVal;
+} SK_APP_CONF;
+
+typedef struct s_AppConfData {
+	int		WakeUpSpeedSel;		/* WakeUp Speed Sel. (10M / 100M / Rev. AN) */
+	int		EnergySaveScheme;	/* Energy Saving Scheme (0..4) */
+	int		LED_SchemeSel;		/* LED Scheme Selection (0..5) */
+	int		AutoNegSchemeSel;	/* Auto-Negotiation Scheme (normal/forced) */
+	SK_BOOL	DynClockGatingEn;	/* Dynamic Clock Gating Enable */
+	SK_BOOL	SmartSpeedDownEn;	/* Smart Speed Down Enable */
+	SK_BOOL	MonitorSpeedStepEn;	/* Monitor Speed Step Enable */
+	SK_BOOL	HW_WakeOnLAN_En;	/* HW Wake-on-LAN Enable */
+	SK_BOOL	WakeFromShutdownEn;	/* Wake From Shutdown (S5) Enable */
+	SK_BOOL	MemoryTypeInternal;	/* Memory Type (internal/external) */
+	SK_BOOL	ValidConfiguration;	/* Valid Configuration Flag */
+} SK_APP_CONFIG;
+
 /*
  * Gigabit Ethernet Initialization Struct
  * (has to be included in the adapter context)
  */
-typedef	struct s_GeInit {
+typedef struct s_GeInit {
 	int			GIChipId;		/* Chip Identification Number */
 	int			GIChipRev;		/* Chip Revision Number */
 	SK_U8		GIPciHwRev;		/* PCI HW Revision Number */
@@ -684,18 +764,18 @@ typedef	struct s_GeInit {
 	SK_BOOL		GIYukon32Bit;	/* 32-Bit YUKON adapter */
 	SK_BOOL		GIAsfEnabled;	/* ASF subsystem enabled */
 	SK_BOOL		GIAsfRunning;	/* ASF subsystem running */
-	SK_BOOL		GIExtLeFormat;	/* Extended list element format (Yukon-Ext) */
+	SK_BOOL		GIExtLeFormat;	/* Extended LE format (Yukon-Ext) */
+	SK_BOOL		GIDontInitPhy;	/* Don't Initialize PHY */
 #ifndef SK_SLIM
-	SK_BOOL		GIJumTcpSegSup;	/* TCP Segmentation of Jumbo frames supported ?*/
+	SK_BOOL		GIJumTcpSegSup;	/* TCP Segmentation of Jumbo frames supp. ? */
 	SK_BOOL		GIGotoD3Cold;	/* System set to D3cold */
 #endif
 	SK_U16		GILedBlinkCtrl;	/* LED Blink Control */
+	SK_U16		GIPexLinkCtrl;	/* PCI-Express Link Control */
 	int			GIMacsFound;	/* Number of MACs found on this adapter */
 	int			GIMacType;		/* MAC Type used on this adapter */
 	int			GIChipCap;		/* Adapter's Capabilities */
-#ifndef SK_SLIM
 	int			GIHwResInfo;	/* HW Resources / Application Information */
-#endif
 	int			GIHstClkFact;	/* Host Clock Factor (HstClk / 62.5 * 100) */
 	int			GILevel;		/* Initialization Level completed */
 	int			GIRamSize;		/* The RAM size of the adapter in kB */
@@ -712,10 +792,13 @@ typedef	struct s_GeInit {
 	SK_U32		GIValHwIrqMask;	/* Value for HWE Interrupt Mask */
 #ifndef SK_SLIM
 	SK_U32		GITimeStampCnt;	/* Time Stamp High Counter (YUKON only) */
+	SK_U32		GIPhyInitTime;	/* Time Stamp on PHY Init */
 #endif
 	SK_GEPORT	GP[SK_MAX_MACS];/* Port Dependent Information */
-	SK_HW_FEATURES HwF;			/* HW Features struct */
-	SK_GEMACFUNC GIFunc;		/* MAC depedent functions */
+	SK_HW_FEATURES	HwF;		/* HW Features struct */
+	SK_APP_CONF		CfgVal;		/* Application Configuration Value */
+	SK_APP_CONFIG	CfgData;	/* Application Configuration Structure */
+	SK_GEMACFUNC	GIFunc;		/* MAC depedent functions */
 } SK_GEINIT;
 
 /*
@@ -776,7 +859,7 @@ typedef	struct s_GeInit {
 #define SKERR_HWI_E027		(SKERR_HWI_E026+1)
 #define SKERR_HWI_E027MSG	""
 
-/* function prototypes ********************************************************/
+/* function prototypes *******************************************************/
 
 #ifndef	SK_KR_PROTO
 
@@ -789,6 +872,12 @@ extern void SkGePortVlan(
 	int		Port,
 	SK_BOOL Enable);
 
+extern void SkGeMacSecLE(
+	SK_AC	*pAC,
+	SK_IOC	IoC,
+	int		Port,
+	SK_BOOL	Enable);
+
 extern void SkGeRxRss(
 	SK_AC	*pAC,
 	SK_IOC	IoC,
@@ -855,12 +944,6 @@ extern int	SkGeInitPort(
 	SK_IOC	IoC,
 	int		Port);
 
-extern void	SkGeXmitLED(
-	SK_AC	*pAC,
-	SK_IOC	IoC,
-	int		Led,
-	int		Mode);
-
 extern void	SkGeInitRamIface(
 	SK_AC	*pAC,
 	SK_IOC	IoC);
@@ -931,16 +1014,6 @@ extern void SkMacIrqDisable(
 	SK_IOC	IoC,
 	int		Port);
 
-extern void	SkMacFlushTxFifo(
-	SK_AC	*pAC,
-	SK_IOC	IoC,
-	int		Port);
-
-extern void	SkMacFlushRxFifo(
-	SK_AC	*pAC,
-	SK_IOC	IoC,
-	int		Port);
-
 extern void	SkMacIrq(
 	SK_AC	*pAC,
 	SK_IOC	IoC,
@@ -1000,11 +1073,6 @@ extern int	SkGmPhyWrite(
 	int		Addr,
 	SK_U16	Val);
 
-extern int SkGmUpdateStats(
-	SK_AC	*pAC,
-	SK_IOC	IoC,
-	unsigned int Port);
-
 extern int SkGmMacStatistic(
 	SK_AC	*pAC,
 	SK_IOC	IoC,
@@ -1024,11 +1092,39 @@ extern int SkGmOverflowStatus(
 	SK_U16	MacStatus,
 	SK_U64	SK_FAR *pStatus);
 
+#ifndef SK_SLIM
+extern int	SkGmPtpRead(
+	SK_AC	*pAC,
+	SK_IOC	IoC,
+	int		Port,
+	int		Addr,
+	SK_U16	*pVal);
+
+extern int	SkGmPtpReadDbl(
+	SK_AC	*pAC,
+	SK_IOC	IoC,
+	int		Port,
+	int		Addr,
+	SK_U32	*pVal);
+
+extern int	SkGmPtpWrite(
+	SK_AC	*pAC,
+	SK_IOC	IoC,
+	int		Port,
+	int		Addr,
+	SK_U16	Val);
+
+extern int SkGmUpdateStats(
+	SK_AC	*pAC,
+	SK_IOC	IoC,
+	unsigned int Port);
+
 extern int SkGmCableDiagStatus(
 	SK_AC	*pAC,
 	SK_IOC	IoC,
 	int		Port,
 	SK_BOOL	StartTest);
+#endif /* !SK_SLIM */
 
 #ifdef SK_PHY_LP_MODE
 extern int SkGmEnterLowPowerMode(
@@ -1095,6 +1191,7 @@ extern void	SkGeInitRamIface();
 extern int	SkGeInitAssignRamToQueues();
 extern void	SkGePortVlan();
 extern void	SkGeRxCsum();
+extern void	SkGeMacSecLE();
 extern void	SkGeRxRss();
 extern void	DoInitRamQueue();
 extern int	SkYuk2RestartRxBmu();
@@ -1113,8 +1210,6 @@ extern int	SkMacRxTxEnable();
 extern void	SkMacPromiscMode();
 extern void	SkMacHashing();
 extern void	SkMacIrqDisable();
-extern void	SkMacFlushTxFifo();
-extern void	SkMacFlushRxFifo();
 extern void	SkMacIrq();
 extern int	SkMacAutoNegDone();
 extern void	SkMacAutoNegLipaPhy();
@@ -1123,11 +1218,18 @@ extern void	SkMacTimeStamp();
 extern void	SkGmInitMac();
 extern int	SkGmPhyRead();
 extern int	SkGmPhyWrite();
-extern int	SkGmUpdateStats();
 extern int	SkGmMacStatistic();
 extern int	SkGmResetCounter();
 extern int	SkGmOverflowStatus();
+
+#ifndef SK_SLIM
+extern int	SkGmPtpRead();
+extern int	SkGmPtpReadDbl();
+extern int	SkGmPtpWrite();
+extern int	SkGmUpdateStats();
 extern int	SkGmCableDiagStatus();
+#endif /* !SK_SLIM */
+
 #ifdef SK_PHY_LP_MODE
 extern int	SkGmEnterLowPowerMode();
 extern int	SkGmLeaveLowPowerMode();
