@@ -26,14 +26,19 @@
  *
  ******************************************************************************/
 
-#define BOOT_STRING  "sk98lin: Network Device Driver v10.70.1.3\n" \
-                     "(C)Copyright 1999-2008 Marvell(R)."
-#define VER_STRING   "10.70.1.3"
-#define PATCHLEVEL   "01"
+#define BOOT_STRING  "sk98lin: Network Device Driver v10.85.8.3\n" \
+                     "(C)Copyright 1999-2010 Marvell(R)."
+#define VER_STRING   "10.85.8.3"
+#define PATCHLEVEL   "08"
 #define DRIVER_FILE_NAME   "sk98lin"
-#define DRIVER_REL_DATE    "Jun-30-2008"
+#define DRIVER_REL_DATE    "Jul-23-2010"
 #define DRV_NAME   "sk98lin"
-#define DRV_VERSION   "10.70.1.3"
+#define DRV_VERSION   "10.85.8.3"
+
+#ifdef MV_INCLUDE_SDK_SUPPORT
+#define FW_FILE_PATHNAME    "/lib/firmware/txbasesu.bin"
+#define FW_FILE_NAME        "txbasesu.bin"
+#endif
 
 /*******************************************************************************
  *
