@@ -3,8 +3,8 @@
 # Part of Marvell Yukon/SysKonnect sk98lin Driver for Linux                   #
 ###############################################################################
 # Installation script for Marvell Chip based Ethernet Gigabit Cards           #
-# $Revision: 1.1.4.24 $                                                       #
-# $Date: 2008/04/09 12:18:00 $                                                #
+# $Revision: 1.6 $                                                       #
+# $Date: 2009/09/04 13:08:48 $                                                #
 # =========================================================================== #
 #                                                                             #
 #  Main - Global function                                                     #
@@ -21,13 +21,17 @@
 # =========================================================================== #
 # COPYRIGHT NOTICE :                                                          #
 #                                                                             #
-# (C)Copyright 2003-2008 Marvell(R).                                          #
+# (C)Copyright 2003-2010 Marvell(R).                                          #
 #                                                                             #
-#  LICENSE:                                                                   #
-#  This program is free software; you can redistribute it                     # 
-#  and/or modify it under the terms of the GNU General Public                 #
-#  License as published by the Free Software Foundation; either               #
-#  version 2 of the License, or (at your option) any later version.           #
+#  LICENSE:
+#  (C)Copyright Marvell.
+#  
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 2 of the License, or
+#  (at your option) any later version.
+#  
+#  The information in this file is provided "AS IS" without warranty.
 #  /LICENSE                                                                   #
 #                                                                             #
 #                                                                             #
@@ -114,7 +118,7 @@ function unpack_driver ()
 	fi
 
 	# Generate all build dir...
-	mkdir ${TMP_DIR}/all
+	mkdir ${TMP_DIR}/all &> /dev/null
 	cp -pr ${TMP_DIR}/common/* ${TMP_DIR}/all
 	cp -pr ${TMP_DIR}/${KERNEL_FAMILY}/* ${TMP_DIR}/all
 }
@@ -123,17 +127,18 @@ function unpack_driver ()
 function help ()
 {
 	echo "Usage: install.sh [OPTIONS]"
-	echo "   Install the sk98lin driver or generate a patch"
+	echo "   Install the ${drv_name} driver or generate a patch"
   	echo "   Example: install.sh"
 	echo
 	echo "Optional parameters:"
-	echo "   -s              install the driver wothout any user interaction"
+	echo "   -s              compile the driver wothout any user interaction"
 	echo "   -c              cleanup all sk98lin temp directories"
 	echo "   -p [KERNEL_DIR] [PATCH_FILE]         generate a patch"
+	echo "   -m              generate a makefile"
 	echo "   -h              display this help and exit"
-	echo "   -v              output version information and exit"
+	echo "   -v              print version information and exit"
 	echo
-	echo "Report bugs to <linux@syskonnect.de>."
+	echo "Report bugs to <support@marvell.com>."
 	exit
 }
 
@@ -147,7 +152,6 @@ function create_makefile ()
 	# Returns:
 	#       N/A
 
-
 	# we have to use the makefile and change the include dir
 	rm -rf ${TMP_DIR}/all/Makefile
  	if [ $1 == "1" ]; then
@@ -158,22 +162,6 @@ function create_makefile ()
 		sed -e "s$A$AFirst$A$ALast$A" \
 			${TMP_DIR}/2.6/Makefile \
 			>> ${TMP_DIR}/all/Makefile
-		if [ $IPMI_SUPPORT == "1" ]; then
-			local IPMIFirst="# ASFPARAM += -DSK_ASF"
-			local IPMILast="ASFPARAM += -DSK_ASF"
-			sed -e "s$A$IPMIFirst$A$IPMILast$A" \
-				${TMP_DIR}/all/Makefile \
-				>> ${TMP_DIR}/all/Makefile2
-			mv ${TMP_DIR}/all/Makefile2 ${TMP_DIR}/all/Makefile
-		fi
-		if [ $DASH_SUPPORT == "1" ]; then
-			local IPMIFirst="# ASFPARAM += -DSK_ASF"
-			local IPMILast="ASFPARAM += -DSK_ASF -DUSE_ASF_DASH_FW"
-			sed -e "s$A$IPMIFirst$A$IPMILast$A" \
-				${TMP_DIR}/all/Makefile \
-				>> ${TMP_DIR}/all/Makefile2
-			mv ${TMP_DIR}/all/Makefile2 ${TMP_DIR}/all/Makefile
-		fi
 	else
 		cp ${TMP_DIR}/2.6/Makefile ${TMP_DIR}/all/Makefile
 	fi
@@ -188,7 +176,19 @@ function create_makefile ()
 			sed -e "s$A$addparamfirst$A$addparam$A" \
 			${TMP_DIR}/all/Makefile \
 			>> ${TMP_DIR}/all/Makefile2
-		cp ${TMP_DIR}/all/Makefile2 ${TMP_DIR}/all/Makefile
+		mv ${TMP_DIR}/all/Makefile2 ${TMP_DIR}/all/Makefile
+	fi
+
+	# Change the makefile for the SDK
+	if [ $SDK_SUPPORT == "1" ]; then
+		# Enable SDK support in the makefile
+		local A="`echo | tr '\012' '\001' `"
+		local SDKFirst="# SDKPARAM += -DMV_INCLUDE_SDK_SUPPORT"
+		local SDKLast="SDKPARAM += -DMV_INCLUDE_SDK_SUPPORT"
+		sed -e "s$A$SDKFirst$A$SDKLast$A" \
+			${TMP_DIR}/all/Makefile \
+			>> ${TMP_DIR}/all/Makefile2
+		mv ${TMP_DIR}/all/Makefile2 ${TMP_DIR}/all/Makefile
 	fi
 
 }
@@ -212,7 +212,12 @@ function make_driver ()
 	message_status 3 "working"
 
 	export CONFIG_SK98LIN=m
-	make $MAKE_CPU_FLAG -C ${KERNEL_SOURCE}  SUBDIRS=${TMP_DIR}/all modules >> $logfile 2>&1
+
+	if [ ${KERNEL_BUILD_OUTPUT} ]; then
+		make $MAKE_CPU_FLAG CC=cc KBUILD_OUTPUT=${KERNEL_BUILD_OUTPUT} KBUILD_VERBOSE=1 -C ${KERNEL_SOURCE} SUBDIRS=${TMP_DIR}/all >> $logfile 2>&1
+	else
+		make $MAKE_CPU_FLAG KBUILD_VERBOSE=1 -C ${KERNEL_SOURCE}  SUBDIRS=${TMP_DIR}/all modules >> $logfile 2>&1
+	fi
 
 	if [ -f $drv_name.ko ]; then
 		cp $drv_name.ko ../
@@ -244,43 +249,13 @@ function install_firmware ()
 	#       N/A
 	echo -n "Check firmware availability"
 
-	if [ -f ${TMP_DIR}/firmware/ipmiyk2-s1.bin ]; then
+	if [ -f ${TMP_DIR}/firmware/txbasesu.bin ]; then
 		message_status 1 "available"
 
-		echo -n "Create /etc/sk98lin directory"
-		if [ -d /etc/sk98lin ]; then
-			message_status 1 "already created"
-		else
-			mkdir /etc/sk98lin
-			message_status 1 "done"
-		fi
-
 		echo -n "Copying firmware"
-		cp ${TMP_DIR}/firmware/ipmiyk2-s1.bin /etc/sk98lin/
-		cp ${TMP_DIR}/firmware/ipmiyk2-s2.bin /etc/sk98lin/
+		cp ${TMP_DIR}/firmware/txbasesu.bin /lib/firmware/
 
-		IPMI_SUPPORT=1
-	else
-		if [ -f ${TMP_DIR}/firmware/dashyex-s1.bin ]; then
-			message_status 1 "available"
-
-			echo -n "Create /etc/sk98lin directory"
-			if [ -d /etc/sk98lin ]; then
-				message_status 1 "already created"
-			else
-				mkdir /etc/sk98lin
-				message_status 1 "done"
-			fi	
-
-			echo -n "Copying firmware"
-			cp ${TMP_DIR}/firmware/dashyex-s1.bin /etc/sk98lin/
-			cp ${TMP_DIR}/firmware/dashyex-s2.bin /etc/sk98lin/
-
-			DASH_SUPPORT=1
-		else
-			message_status 1 "not available"
-			return
-		fi
+#		SDK_SUPPORT=1
 	fi
 
 	message_status 1 "done"
@@ -452,9 +427,10 @@ function get_patch_infos ()
 
 	if [ ! ${INSTALL_AUTO} ]; then
 		drvvertmp=`echo $DRIVER_VERSION | sed -e "s/ //g"`
+		drvvertmp=`echo $drvvertmp | cut -d '(' -f 1`	
 		PATCH_NAME="$working_dir/sk98lin_v"
 		PATCH_NAME="$PATCH_NAME$drvvertmp"
-		PATCH_NAME="${PATCH_NAME}_${KERNEL_VERSION}_patch"
+		PATCH_NAME="${PATCH_NAME}_K${KERNEL_VERSION}.patch"
 
 		echo -n "Patch name ($PATCH_NAME) : "
 		read patch_name_in
@@ -494,23 +470,36 @@ function generate_patch_for_kconfig ()
 
 	# find the first line of the sk block
 	startline=`grep "SK98LIN$" $kerneldir/drivers/net/Kconfig -n | grep -v "depends" | cut -d ':' -f 1`
-	totalline=`cat $kerneldir/drivers/net/Kconfig | wc -l`
-	((startline=$startline - 1))
-	((splitline=$totalline - $startline))
-	((splitline=$splitline - 2))
-	head -n $startline $kerneldir/drivers/net/Kconfig > ${TMP_DIR}/Kconfig
-	cat $packagedir/misc/Kconfig >> ${TMP_DIR}/Kconfig
-	tail -n $splitline $kerneldir/drivers/net/Kconfig > ${TMP_DIR}/Kconfigtemp
 
-	# find the end of the block	
-	while read line; do
-		((count=$count + 1))
-		splt=`echo $line | grep "^config"  -c`
-		if [ $splt -gt 0 ]; then break; fi
-	done < ${TMP_DIR}/Kconfigtemp
-	((count=$count - 2))
-	((splitline=$splitline - $count))
-	tail -n $splitline $kerneldir/drivers/net/Kconfig >> ${TMP_DIR}/Kconfig
+	if [ ! ${startline} ]; then
+		# Probably a new kernel... Check for SKGE
+		startline=`grep "SKGE$" $kerneldir/drivers/net/Kconfig -n | grep -v "depends" | cut -d ':' -f 1`
+		totalline=`cat $kerneldir/drivers/net/Kconfig | wc -l`
+		((startline=$startline - 1))
+		((splitline=$totalline - $startline))
+		head -n $startline $kerneldir/drivers/net/Kconfig > ${TMP_DIR}/Kconfig
+		cat $packagedir/misc/Kconfig >> ${TMP_DIR}/Kconfig
+		tail -n $splitline $kerneldir/drivers/net/Kconfig >> ${TMP_DIR}/Kconfig
+	else
+		# Old kernel version. Replace the sk98lin driver
+		totalline=`cat $kerneldir/drivers/net/Kconfig | wc -l`
+		((startline=$startline - 1))
+		((splitline=$totalline - $startline))
+		((splitline=$splitline - 2))
+		head -n $startline $kerneldir/drivers/net/Kconfig > ${TMP_DIR}/Kconfig
+		cat $packagedir/misc/Kconfig >> ${TMP_DIR}/Kconfig
+		tail -n $splitline $kerneldir/drivers/net/Kconfig > ${TMP_DIR}/Kconfigtemp
+
+		# find the end of the block	
+		while read line; do
+			((count=$count + 1))
+			splt=`echo $line | grep "^config"  -c`
+			if [ $splt -gt 0 ]; then break; fi
+		done < ${TMP_DIR}/Kconfigtemp
+		((count=$count - 2))
+		((splitline=$splitline - $count))
+		tail -n $splitline $kerneldir/drivers/net/Kconfig >> ${TMP_DIR}/Kconfig
+	fi
 
 	# Make diff
 	diff -ruN $kerneldir/drivers/net/Kconfig ${TMP_DIR}/Kconfig \
@@ -534,6 +523,61 @@ linux-new/drivers/net/Kconfig" >> ${PATCH_NAME}
 	message_status 1 "done"
 }
 
+function generate_patch_for_makefile ()
+{
+	# Generate a patch for the config file
+	# Syntax: generate_patch_for_makefile
+	#	package_root, kernel_root
+	# Author: mlindner
+	# Returns:
+	#       N/A
+	fname="Generate Makefile patch"
+	echo -n $fname
+	message_status 3 "working"
+
+	local packagedir="$1"
+	local kerneldir=$2
+	local startline
+	local totalline
+	local splitline
+	local count=0
+	local splt=0
+	local A="`echo | tr '\012' '\001' `"
+
+	# find the first line of the sk block
+	startline=`grep "SK98LIN" $kerneldir/drivers/net/Makefile -n | cut -d ':' -f 1`
+
+	if [ ! ${startline} ]; then
+		# Probably a new kernel... Check for SKY2
+		startline=`grep "SKY2" $kerneldir/drivers/net/Makefile -n | cut -d ':' -f 1`
+		totalline=`cat $kerneldir/drivers/net/Makefile| wc -l`
+		((splitline=$totalline - $startline))
+		head -n $startline $kerneldir/drivers/net/Makefile > ${TMP_DIR}/Makefile
+
+		echo "obj-\$(CONFIG_SK98LIN) += sk98lin/">> ${TMP_DIR}/Makefile
+		tail -n $splitline $kerneldir/drivers/net/Makefile >> ${TMP_DIR}/Makefile
+
+		# Make diff
+		diff -ruN $kerneldir/drivers/net/Makefile ${TMP_DIR}/Makefile \
+			> ${TMP_DIR}/all_sources_patch
+		replacement="linux-new/drivers/net"
+		sed -e "s$A${TMP_DIR}$A$replacement$A" \
+			${TMP_DIR}/all_sources_patch &> ${TMP_DIR}/all_sources_patch2
+		replacement="linux"
+		sed -e "s$A$kerneldir$A$replacement$A" \
+			${TMP_DIR}/all_sources_patch2 &> ${TMP_DIR}/all_sources_patch
+
+		# Complete the patch
+		echo "diff -ruN linux/drivers/net/Makefile \
+linux-new/drivers/net/Makefile" >> ${PATCH_NAME}
+		cat ${TMP_DIR}/all_sources_patch >> ${PATCH_NAME}
+	fi
+
+	# Status
+	echo -en "\015"
+	echo -n $fname
+	message_status 1 "done"
+}
 
 function generate_patch_for_readme ()
 {
@@ -633,6 +677,75 @@ function generate_patch_file ()
 	message_status 1 "done"
 }
 
+function makefile_generation ()
+{
+	# Generate a makefile version
+	# Syntax: makefile_generation
+	# Author: mlindner
+	# Returns:
+	#       N/A
+
+	# Create tmp dir and unpack the driver
+	KERNEL_FAMILY=2.6
+	TMP_DIR=src
+	working_dir=`pwd`
+	rm -rf src
+	mkdir src &> /dev/null
+
+	if [ $SDK_SUPPORT == "0" ]; then
+		check_kernel_informations
+	fi
+
+	unpack_driver
+
+	# Cleanup
+	rm -rf src/sk98*
+	rm -rf src/2.6
+	rm -rf src/common
+	rm -rf src/misc
+	mv src/all/* src
+	rm -rf src/all
+	local A="`echo | tr '\012' '\001' `"
+
+	# Change the makefile
+	if [ $SDK_SUPPORT == "1" ]; then
+		# Enable SDK support in the makefile
+		local SDKFirst="# SDKPARAM += -DMV_INCLUDE_SDK_SUPPORT"
+		local SDKLast="SDKPARAM += -DMV_INCLUDE_SDK_SUPPORT"
+		sed -e "s$A$SDKFirst$A$SDKLast$A" \
+			src/Makefile >> src/Makefile2
+		mv src/Makefile2 src/Makefile
+	fi
+
+	# Set the src directory
+	local AFirst="Idrivers/net/sk98lin"
+	local ALast="I${working_dir}/src"
+	sed -e "s$A$AFirst$A$ALast$A" \
+		src/Makefile >> src/Makefile2
+	mv src/Makefile2 src/Makefile
+
+	# Check for make command
+	if [ ${KERNEL_BUILD_OUTPUT} ]; then
+		make_command="make $MAKE_CPU_FLAG CC=cc KBUILD_OUTPUT=${KERNEL_BUILD_OUTPUT} -C ${KERNEL_SOURCE} SUBDIRS=${working_dir}/src"
+	else
+		make_command="make $MAKE_CPU_FLAG -C ${KERNEL_SOURCE} SUBDIRS=${working_dir}/src modules"
+	fi
+
+	clear
+
+	# Print the makefile message
+	echo
+	echo
+	echo "All done. Makefile version successfully generated."
+	if [ $SDK_SUPPORT == "0" ]; then
+		echo "To generate the driver, proceed as follows:"
+		echo "  # export CONFIG_SK98LIN=m"
+		echo "  # $make_command"
+	fi
+	echo 
+	echo "                                                     Have fun..."
+	exit
+}
 
 function patch_generation ()
 {
@@ -672,6 +785,9 @@ function patch_generation ()
 	# Generate a patch for the config file
 	generate_patch_for_kconfig ${TMP_DIR} $KERNEL_SOURCE
 
+	# Generate a patch for the Makefile
+	generate_patch_for_makefile ${TMP_DIR} $KERNEL_SOURCE
+	
 	# Clear the tmp dirs
 	clean
 
@@ -722,7 +838,12 @@ function start_sequence ()
 	echo 
 	echo "Installation script for $drv_name driver."
 	echo "Version $VERSION"
-	echo "(C)Copyright 2003-2008 Marvell(R)."
+	echo "(C)Copyright 2003-2010 Marvell(R)."
+
+	if [ $OPTION_VERSION ]; then
+		exit;
+	fi
+
 	echo "===================================================="
 	echo "Add to your trouble-report the logfile install.log"
 	echo "which is located in the  DriverInstall directory."
@@ -738,7 +859,7 @@ function start_sequence ()
 
 	echo
 
-	select user_sel in "installation" "generate patch" "exit"
+	select user_sel in "installation" "generate patch" "generate makefile" "exit"
 	do
 		break  # if no 'break' here, keeps looping forever.
 	done
@@ -748,15 +869,23 @@ function start_sequence ()
 		exit
 	fi
 
-	if [ "$user_sel" != "installation" ] && [ "$user_sel" != "expert installation" ] && [ "$user_sel" != "generate patch" ]; then
+	if [ "$user_sel" != "installation" ] && [ "$user_sel" != "expert installation" ] && [ "$user_sel" != "generate patch" ] && [ "$user_sel" != "generate makefile" ]; then
 		echo "Exit."
 		exit
 	fi
 
 	clear
 
+
+	if [ "$user_sel" == "generate makefile" ]; then 
+		export REMOVE_SKDEVICE=0
+		INSTALL_MODE="MAKEFILE"
+		makefile_generation
+		exit
+	fi
+
 	if [ "$user_sel" == "installation" ]; then
-		echo "Please read this carfully!"
+		echo "Please read this carefully!"
 		echo
 		echo "This script will automatically compile and load the $drv_name"
 		echo "driver on your host system. Before performing both compilation"
@@ -789,7 +918,6 @@ function start_sequence ()
 				exit 0
 			fi
 		fi
-		
 		export REMOVE_SKDEVICE=1
 
 	else
@@ -808,7 +936,7 @@ function start_sequence ()
 function main_global ()
 {
 	# Main function
-	# Syntax: patch_generation
+	# Syntax: main_global
 	# Author: mlindner
 	# Returns:
 	#       N/A
@@ -817,6 +945,7 @@ function main_global ()
 	extract_params $*
 
 	# Run given functions
+	if [ "${OPTION_SDK}" ]; then SDK_SUPPORT=1; fi
 	if [ "${OPTION_HELP}" ]; then help; fi
 	if [ "${OPTION_SILENT}" ]; then user_sel=`echo user`; fi
 	if [ "${OPTION_LOG}" ]; then print_comp_resources; exit 0; fi
@@ -832,6 +961,13 @@ function main_global ()
 		fi
 	fi
 
+	if [ "${OPTION_MAKEFILE}" ]; then
+		echo generate makefile
+		INSTALL_MODE="MAKEFILE"
+		makefile_generation
+		exit 0
+	fi
+
 	# Delete log file
 	rm -rf $logfile &> /dev/null
 
@@ -885,7 +1021,7 @@ function main_global ()
 #####################################################################
 
 drv_name=`echo sk98lin`
-VERSION=`echo "10.70.1.3 (Jun-30-2008)"`
+VERSION=`echo "10.85.8.3 (Jul-23-2010)"`
 #drv_name=`echo sk98lin`
 working_dir=`pwd`
 logfile="$working_dir/install.log"
@@ -893,8 +1029,7 @@ trap cleanup_trap INT TERM
 KERNEL_TREE_MAKE=0;
 ALT_OPTION_FLAG=0
 ALT_OPTION=0
-IPMI_SUPPORT=0
-DASH_SUPPORT=0
+SDK_SUPPORT=0
 clear
 
 # Run main function
@@ -902,4 +1037,3 @@ main_global $*
 
 # Exit
 exit 0
-
