@@ -1,22 +1,22 @@
 /******************************************************************************
  *
  * Name:        skethtool.c
- * Project:     GEnesis, PCI Gigabit Ethernet Adapter
- * Version:     $Revision: 1.9.2.10 $
- * Date:        $Date: 2008/04/09 12:16:01 $
+ * Project:     Yukon/Yukon2, PCI Gigabit Ethernet Adapter
+ * Version:     $Revision: 1.12 $
+ * Date:        $Date: 2009/05/07 09:04:18 $
  * Purpose:     All functions regarding ethtool handling
  *
  ******************************************************************************/
 
 /******************************************************************************
  *
- *	(C)Copyright 1998-2002 SysKonnect GmbH.
- *	(C)Copyright 2002-2007 Marvell.
+ *	    (C)Copyright 1998-2002 SysKonnect GmbH.
+ *	    (C)Copyright 2002-2009 Marvell.
  *
- *	Driver for Marvell Yukon/2 chipset and SysKonnect Gigabit Ethernet 
+ *	    Driver for Marvell Yukon/2 chipset and SysKonnect Gigabit Ethernet 
  *      Server Adapters.
  *
- *	Address all question to: gr-msgg-linux@marvell.com
+ *	    Address all question to: support@marvell.com
  *
  *      LICENSE:
  *      (C)Copyright Marvell.
@@ -190,6 +190,7 @@ static SK_BOOL doSwitchLEDsOn       = SK_FALSE;
 static SK_BOOL boardWasDown[2]      = { SK_FALSE, SK_FALSE };
 static struct timer_list locateNICtimer;
 
+
 /******************************************************************************
  *
  * Ethtool Functions
@@ -211,7 +212,7 @@ static struct timer_list locateNICtimer;
 int SkGeGetSettings(struct net_device *dev,
 			struct ethtool_cmd *ecmd)
 {
-	DEV_NET			*pNet        = (DEV_NET*) dev->priv;
+	DEV_NET			*pNet        = PPRIV;
 	SK_AC			*pAC         = pNet->pAC;
 	int			port         = pNet->PortNr;
 	SK_GEPORT		*pPort       = &pAC->GIni.GP[port];
@@ -274,7 +275,7 @@ int SkGeGetSettings(struct net_device *dev,
 void SkGeGetDrvInfo(struct net_device *dev,
 			struct ethtool_drvinfo *ecmd)
 {
-	DEV_NET			*pNet        = (DEV_NET*) dev->priv;
+	DEV_NET			*pNet        = PPRIV;
 	SK_AC			*pAC         = pNet->pAC;
 	char			versionString[32];
 
@@ -301,7 +302,7 @@ void SkGeGetDrvInfo(struct net_device *dev,
 void SkGeGetWolSettings(struct net_device *dev,
 			struct ethtool_wolinfo *ecmd)
 {
-	DEV_NET			*pNet        = (DEV_NET*) dev->priv;
+	DEV_NET			*pNet        = PPRIV;
 	SK_AC			*pAC         = pNet->pAC;
 
 	ecmd->supported = pAC->WolInfo.SupportedWolOptions;
@@ -322,7 +323,7 @@ void SkGeGetWolSettings(struct net_device *dev,
 void SkGeGetPauseParam(struct net_device *dev,
 			struct ethtool_pauseparam *ecmd)
 {
-	DEV_NET			*pNet        = (DEV_NET*) dev->priv;
+	DEV_NET			*pNet        = PPRIV;
 	SK_AC			*pAC         = pNet->pAC;
 	int			port         = pNet->PortNr;
 	SK_GEPORT		*pPort       = &pAC->GIni.GP[port];
@@ -362,7 +363,7 @@ void SkGeGetPauseParam(struct net_device *dev,
 int SkGeGetCoalesce(struct net_device *dev,
 			struct ethtool_coalesce *ecmd)
 {
-	DEV_NET			*pNet        = (DEV_NET*) dev->priv;
+	DEV_NET			*pNet        = PPRIV;
 	SK_AC			*pAC         = pNet->pAC;
 
 	DIM_INFO *Info = &pAC->DynIrqModInfo;
@@ -427,7 +428,7 @@ int SkGeGetCoalesce(struct net_device *dev,
 SK_U32 SkGeGetRxCsum(struct net_device *dev)
 {
 
-	DEV_NET			*pNet        = (DEV_NET*) dev->priv;
+	DEV_NET			*pNet        = PPRIV;
 	SK_AC			*pAC         = pNet->pAC;
 	int			port         = pNet->PortNr;
 
@@ -446,12 +447,12 @@ SK_U32 SkGeGetRxCsum(struct net_device *dev)
  *
  */
 void SkGeGetStrings(struct net_device *dev, 
-			u32 stringset,
-			u8 *strings)
+	u32 stringset,
+	u8 *strings)
 {
-	DEV_NET			*pNet        = (DEV_NET*) dev->priv;
-	int			port         = pNet->PortNr;
-	int			i;
+	DEV_NET	*pNet     = PPRIV;
+	int	port      = pNet->PortNr;
+	int	i;
 
 #if LINUX_VERSION_CODE > KERNEL_VERSION(2,4,19)
 	struct sk98lin_stats *sk98lin_etht_stats = 
@@ -486,8 +487,25 @@ int SkGeGetStatsLen(struct net_device *dev)
 	return SK98LIN_STATS_LEN;
 }
 
-
-
+/*****************************************************************************
+ *
+ * 	 SkGeGetSsetCount - retrieves the new statistic count
+ *
+ * Description:
+ *	N/A
+ *
+ * Returns:	N/A
+ *
+ */
+int SkGeGetSsetCount(struct net_device *dev, int sset)
+{
+        switch (sset) {
+        case ETH_SS_STATS:
+                return SK98LIN_STATS_LEN;
+        default:
+                return -EOPNOTSUPP;
+        }
+}
 
 /*****************************************************************************
  *
@@ -504,7 +522,7 @@ void SkGeGetEthStats(struct net_device *dev,
 			struct ethtool_stats *stats,
 			u64 *data)
 {
-	DEV_NET			*pNet        = (DEV_NET*) dev->priv;
+	DEV_NET			*pNet        = PPRIV;
 	SK_AC			*pAC         = pNet->pAC;
 	SK_U32			Size         = sizeof(SK_PNMI_STRUCT_DATA);
 	SK_PNMI_STRUCT_DATA     *pPnmiStruct = &pAC->PnmiStruct;
@@ -549,7 +567,7 @@ void SkGeGetEthStats(struct net_device *dev,
 int SkGeSetSettings(struct net_device *dev,
 			struct ethtool_cmd *ecmd)
 {
-	DEV_NET			*pNet        = (DEV_NET*) dev->priv;
+	DEV_NET			*pNet        = PPRIV;
 	SK_AC			*pAC         = pNet->pAC;
 	int			port         = pNet->PortNr;
 
@@ -637,7 +655,7 @@ int SkGeSetSettings(struct net_device *dev,
 int SkGeSetWolSettings(struct net_device *dev,
 			struct ethtool_wolinfo *ecmd)
 {
-	DEV_NET			*pNet        = (DEV_NET*) dev->priv;
+	DEV_NET			*pNet        = PPRIV;
 	SK_AC			*pAC         = pNet->pAC;
 
 	if (ecmd->wolopts != WAKE_MAGIC && ecmd->wolopts != 0)
@@ -665,7 +683,7 @@ int SkGeSetWolSettings(struct net_device *dev,
 int SkGeSetPauseParam(struct net_device *dev,
 			struct ethtool_pauseparam *ecmd)
 {
-	DEV_NET			*pNet        = (DEV_NET*) dev->priv;
+	DEV_NET			*pNet        = PPRIV;
 	SK_AC			*pAC         = pNet->pAC;
 	int			port         = pNet->PortNr;
 	SK_GEPORT		*pPort = &pAC->GIni.GP[port];
@@ -773,7 +791,7 @@ int SkGeSetPauseParam(struct net_device *dev,
 int SkGeSetCoalesce(struct net_device *dev,
 			struct ethtool_coalesce *ecmd)
 {
-	DEV_NET			*pNet        = (DEV_NET*) dev->priv;
+	DEV_NET			*pNet        = PPRIV;
 	SK_AC			*pAC         = pNet->pAC;
 	DIM_INFO		*Info        = &pAC->DynIrqModInfo;
 	int			PrevModeration = Info->IntModTypeSelect;
@@ -957,7 +975,7 @@ int SkGeSetTxCsum(struct net_device *dev,
 int SkGeSetRxCsum(struct net_device *dev,
 			u32 data)
 {
-	DEV_NET			*pNet        = (DEV_NET*) dev->priv;
+	DEV_NET			*pNet        = PPRIV;
 	SK_AC			*pAC         = pNet->pAC;
 	int			port         = pNet->PortNr;
 
@@ -982,7 +1000,7 @@ int SkGeSetRxCsum(struct net_device *dev,
 int SkGePhysId(struct net_device *dev,
 			u32 data)
 {
-	DEV_NET			*pNet        = (DEV_NET*) dev->priv;
+	DEV_NET			*pNet        = PPRIV;
 	SK_AC			*pAC         = pNet->pAC;
 	SK_IOC                   IoC       = pAC->IoBase;
 	int			port         = pNet->PortNr;
@@ -1001,7 +1019,11 @@ int SkGePhysId(struct net_device *dev,
 	if (netif_running(pAC->dev[port])) {
 		boardWasDown[0] = SK_FALSE;
 	} else {
+#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,29)
+		(*pDev->netdev_ops->ndo_open)(pDev);
+#else
 		(*pDev->open)(pDev);
+#endif
 		boardWasDown[0] = SK_TRUE;
 	}
 
@@ -1009,7 +1031,11 @@ int SkGePhysId(struct net_device *dev,
 		if (netif_running(pAC->dev[OtherPort])) {
 			boardWasDown[1] = SK_FALSE;
 		} else {
+#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,29)
+			(*pOtherDev->netdev_ops->ndo_open)(pOtherDev);
+#else
 			(*pOtherDev->open)(pOtherDev);
+#endif
 			boardWasDown[1] = SK_TRUE;
 		}
 	}
@@ -1086,7 +1112,11 @@ unsigned long ptr)  /* holds the pointer to adapter control context */
 	     * The board is already up as we bring it up in case it is not.
 	     */
 	  } else {
+#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,29)
+	    (*pDev->netdev_ops->ndo_stop)(pDev);
+#else
 	    (*pDev->stop)(pDev);
+#endif
 	  }
 	  if (isDualNetCard) {
 	    if (!boardWasDown[1]) {
@@ -1094,7 +1124,11 @@ unsigned long ptr)  /* holds the pointer to adapter control context */
 	       * The board is already up as we bring it up in case it is not.
 	       */
 	    } else {
+#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,29)
+	      (*pOtherDev->netdev_ops->ndo_stop)(pOtherDev);
+#else
 	      (*pOtherDev->stop)(pOtherDev);
+#endif
 	    }
 	    
 	  }
@@ -1184,7 +1218,7 @@ unsigned long ptr)  /* holds the pointer to adapter control context */
 int SkGeSetTSO(struct net_device *dev,
 			u32 data)
 {
-	DEV_NET			*pNet        = (DEV_NET*) dev->priv;
+	DEV_NET			*pNet        = PPRIV;
 	SK_AC			*pAC         = pNet->pAC;
 
 	if (CHIP_ID_YUKON_2(pAC)) {
